@@ -28,9 +28,12 @@ import logging
 import shutil
 import os
 import pprint
+import configparser
+from git.config import GitConfigParser
 from .libkas import (ssh_cleanup_agent, ssh_setup_agent, ssh_no_host_key_check,
                      get_build_environ, repos_fetch, repos_apply_patches)
 from .includehandler import IncludeException
+from .kasusererror import ArgsCombinationError
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2018'
@@ -40,6 +43,7 @@ class Macro:
     """
         Contains commands and provides method to run them.
     """
+
     def __init__(self, use_common_setup=True, use_common_cleanup=True):
         if use_common_setup:
             repo_loop = Loop('repo_setup_loop')
@@ -51,6 +55,10 @@ class Macro:
 
             if ('SSH_PRIVATE_KEY' in os.environ
                     or 'SSH_PRIVATE_KEY_FILE' in os.environ):
+                if 'SSH_AUTH_SOCK' in os.environ:
+                    raise ArgsCombinationError(
+                        'Internal SSH agent (e.g. for "SSH_PRIVATE_KEY") can '
+                        'only be started if no external one is passed.')
                 self.setup_commands.append(SetupSSHAgent())
 
             self.setup_commands += [
@@ -58,6 +66,7 @@ class Macro:
                 InitSetupRepos(),
                 repo_loop,
                 FinishSetupRepos(),
+                ReposCheckout(),
                 ReposApplyPatches(),
                 SetupEnviron(),
                 WriteBBConfig(),
@@ -114,6 +123,7 @@ class Loop(Command):
     """
         A class that defines a set of commands as a loop.
     """
+
     def __init__(self, name):
         self.commands = []
         self.name = name
@@ -152,8 +162,11 @@ class SetupHome(Command):
     ENV_VARS = [
         'GIT_CREDENTIAL_HELPER',
         'GIT_CREDENTIAL_USEHTTPPATH',
+        'GITCONFIG_FILE',
         'AWS_CONFIG_FILE',
+        'AWS_ROLE_ARN',
         'AWS_SHARED_CREDENTIALS_FILE',
+        'AWS_WEB_IDENTITY_TOKEN_FILE',
         'NETRC_FILE',
     ]
 
@@ -167,6 +180,60 @@ class SetupHome(Command):
     def __str__(self):
         return 'setup_home'
 
+    def _setup_aws_creds(self):
+        aws_dir = self.tmpdirname + "/.aws"
+        conf_file = aws_dir + "/config"
+        shared_creds_file = aws_dir + "/credentials"
+        os.makedirs(aws_dir)
+        if os.environ.get('AWS_CONFIG_FILE') \
+                and os.environ.get('AWS_SHARED_CREDENTIALS_FILE'):
+            shutil.copy(os.environ['AWS_CONFIG_FILE'], conf_file)
+            shutil.copy(os.environ['AWS_SHARED_CREDENTIALS_FILE'],
+                        shared_creds_file)
+
+        # OAuth 2.0 workflow credentials
+        if os.environ.get('AWS_WEB_IDENTITY_TOKEN_FILE') \
+                and os.environ.get('AWS_ROLE_ARN'):
+            webid_token_file = aws_dir + '/web_identity_token'
+            config = configparser.ConfigParser()
+            if os.path.exists(conf_file):
+                config.read(conf_file)
+            if 'default' not in config:
+                config['default'] = {}
+            config['default']['role_arn'] = os.environ.get('AWS_ROLE_ARN')
+            config['default']['web_identity_token_file'] = webid_token_file
+            with open(aws_dir + '/config', 'w') as fds:
+                config.write(fds)
+            shutil.copy(os.environ['AWS_WEB_IDENTITY_TOKEN_FILE'],
+                        webid_token_file)
+
+    def _setup_gitconfig(self):
+        gitconfig_host = os.environ.get('GITCONFIG_FILE', False)
+        gitconfig_kas = self.tmpdirname + '/.gitconfig'
+
+        # when running in the github ci, always try to read the gitconfig
+        if not gitconfig_host and \
+           os.environ.get('GITHUB_ACTIONS', False) == 'true':
+            gitconfig_host = os.path.expanduser('~/.gitconfig')
+
+        if gitconfig_host and os.path.exists(gitconfig_host):
+            shutil.copy(gitconfig_host, gitconfig_kas)
+
+        with GitConfigParser(gitconfig_kas, read_only=False) as config:
+            # overwrite user as kas operates git
+            config['user'] = {
+                'email': 'kas@example.com',
+                'name': 'Kas User'
+            }
+            if os.environ.get('GIT_CREDENTIAL_HELPER', False):
+                config['credential'] = {
+                    'helper': os.environ.get('GIT_CREDENTIAL_HELPER')
+                }
+                if os.environ.get('GIT_CREDENTIAL_USEHTTPPATH', False):
+                    config['credential']['useHttpPath'] = \
+                        os.environ.get('GIT_CREDENTIAL_USEHTTPPATH')
+            config.write()
+
     def execute(self, ctx):
         if os.environ.get('NETRC_FILE', False):
             shutil.copy(os.environ['NETRC_FILE'],
@@ -178,28 +245,9 @@ class SetupHome(Command):
                 fds.write('machine ' + os.environ['CI_SERVER_HOST'] + '\n'
                           'login gitlab-ci-token\n'
                           'password ' + os.environ['CI_JOB_TOKEN'] + '\n')
-        with open(self.tmpdirname + '/.gitconfig', 'w') as fds:
-            fds.write('[User]\n'
-                      '\temail = kas@example.com\n'
-                      '\tname = Kas User\n')
-            if os.environ.get('GIT_CREDENTIAL_HELPER', False):
-                fds.write('[credential]\n'
-                          '\thelper = '
-                          + os.environ.get('GIT_CREDENTIAL_HELPER')
-                          + '\n')
-                if os.environ.get('GIT_CREDENTIAL_USEHTTPPATH', False):
-                    fds.write('\tuseHttpPath = '
-                              + os.environ.get('GIT_CREDENTIAL_USEHTTPPATH')
-                              + '\n')
 
-        if os.environ.get('AWS_CONFIG_FILE', False) \
-                and os.environ.get('AWS_SHARED_CREDENTIALS_FILE', False):
-            os.makedirs(self.tmpdirname + "/.aws")
-
-            shutil.copy(os.environ['AWS_CONFIG_FILE'],
-                        self.tmpdirname + "/.aws/config")
-            shutil.copy(os.environ['AWS_SHARED_CREDENTIALS_FILE'],
-                        self.tmpdirname + "/.aws/credentials")
+        self._setup_gitconfig()
+        self._setup_aws_creds()
 
         ctx.environ['HOME'] = self.tmpdirname
 
@@ -227,6 +275,11 @@ class SetupSSHAgent(Command):
         return 'setup_ssh_agent'
 
     def execute(self, ctx):
+        ssh_tools = ['ssh', 'ssh-add', 'ssh-agent']
+        for tool in ssh_tools:
+            if shutil.which(tool) is None:
+                raise RuntimeError('SSH setup requested but could '
+                                   f'not find "{tool}" in PATH')
         ssh_setup_agent()
         ssh_no_host_key_check()
 
@@ -293,29 +346,15 @@ class WriteBBConfig(Command):
             filename = ctx.build_dir + '/conf/local.conf'
             with open(filename, 'w') as fds:
                 fds.write(ctx.config.get_local_conf_header())
-                fds.write('MACHINE ??= "{}"\n'.format(
-                    ctx.config.get_machine()))
-                fds.write('DISTRO ??= "{}"\n'.format(
-                    ctx.config.get_distro()))
-                fds.write('BBMULTICONFIG ?= "{}"\n'.format(
-                    ctx.config.get_multiconfig()))
+                fds.write(f'MACHINE ??= "{ctx.config.get_machine()}"\n')
+                fds.write(f'DISTRO ??= "{ctx.config.get_distro()}"\n')
+                fds.write('BBMULTICONFIG ?= '
+                          f'"{ctx.config.get_multiconfig()}"\n')
 
         _write_bblayers_conf(ctx)
         _write_local_conf(ctx)
 
 
-class ReposFetch(Command):
-    """
-        Fetches repositories defined in the configuration
-    """
-
-    def __str__(self):
-        return 'repos_fetch'
-
-    def execute(self, ctx):
-        repos_fetch(ctx.config.get_repos())
-
-
 class ReposApplyPatches(Command):
     """
         Applies the patches defined in the configuration to the repositories.
@@ -328,19 +367,6 @@ class ReposApplyPatches(Command):
         repos_apply_patches(ctx.config.get_repos())
 
 
-class ReposCheckout(Command):
-    """
-        Ensures that the right revision of each repo is checked out.
-    """
-
-    def __str__(self):
-        return 'repos_checkout'
-
-    def execute(self, ctx):
-        for repo in ctx.config.get_repos():
-            repo.checkout()
-
-
 class InitSetupRepos(Command):
     """
         Prepares setting up repos including the include logic
@@ -368,7 +394,8 @@ class SetupReposStep(Command):
 
         if ctx.missing_repo_names == ctx.missing_repo_names_old:
             raise IncludeException('Could not fetch all repos needed by '
-                                   'includes.')
+                                   'includes. Missing repos: {}'
+                                   .format(', '.join(ctx.missing_repo_names)))
 
         logging.debug('Missing repos for complete config:\n%s',
                       pprint.pformat(ctx.missing_repo_names))
@@ -376,8 +403,8 @@ class SetupReposStep(Command):
         ctx.missing_repos = []
         for repo_name in ctx.missing_repo_names:
             if repo_name not in ctx.config.get_repos_config():
-                raise IncludeException('Include references unknown repo: {}'
-                                       .format(repo_name))
+                # we don't have this repo yet (e.g. due to transitive incl.)
+                continue
             ctx.missing_repos.append(ctx.config.get_repo(repo_name))
 
         repos_fetch(ctx.missing_repos)
@@ -407,11 +434,21 @@ class FinishSetupRepos(Command):
         return 'finish_setup_repos'
 
     def execute(self, ctx):
-        # now fetch everything with complete config and check out layers
+        # now fetch everything with complete config
         repos_fetch(ctx.config.get_repos())
 
-        for repo in ctx.config.get_repos():
-            repo.checkout()
-
         logging.debug('Configuration from config file:\n%s',
                       pprint.pformat(ctx.config.get_config()))
+
+
+class ReposCheckout(Command):
+    """
+        Ensures that the right revision of each repo is checked out.
+    """
+
+    def __str__(self):
+        return 'repos_checkout'
+
+    def execute(self, ctx):
+        for repo in ctx.config.get_repos():
+            repo.checkout()
