@@ -61,7 +61,7 @@ class TaskExecError(KasUserError):
     """
     def __init__(self, command, ret_code):
         self.ret_code = ret_code
-        super().__init__('{} failed: error code {}'.format(command, ret_code))
+        super().__init__(f'{command} failed: error code {ret_code}')
 
 
 class LogOutput:
@@ -108,14 +108,14 @@ async def _read_stream(stream, callback):
             break
 
 
-async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=True):
+async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=False):
     """
         Run a command asynchronously.
     """
 
     env = env or get_context().environ
     cmdstr = ' '.join(cmd)
-    logging.info('%s$ %s', cwd, cmdstr)
+    logging.debug('%s$ %s', cwd, cmdstr)
 
     logo = LogOutput(liveupdate)
 
@@ -150,7 +150,7 @@ async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=True):
     ret = await process.wait()
 
     if ret and fail:
-        msg = 'Command "{cwd}$ {cmd}" failed'.format(cwd=cwd, cmd=cmdstr)
+        msg = f'Command "{cwd}$ {cmdstr}" failed'
         if logo.stderr:
             msg += '\n--- Error summary ---\n'
             for line in logo.stderr:
@@ -161,7 +161,7 @@ async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=True):
     return (ret, ''.join(logo.stdout))
 
 
-def run_cmd(cmd, cwd, env=None, fail=True, liveupdate=True):
+def run_cmd(cmd, cwd, env=None, fail=True, liveupdate=False):
     """
         Runs a command synchronously.
     """
@@ -242,9 +242,9 @@ def get_build_environ(build_system):
         if os.path.exists(repo.path + '/' + script):
             if init_repo:
                 raise InitBuildEnvError(
-                    'Multiple init scripts found ({} vs. {}). '
-                    'Resolve ambiguity by removing one of the repos'
-                    .format(repo.name, init_repo.name))
+                    'Multiple init scripts found '
+                    f'({repo.name} vs. {init_repo.name}). '
+                    'Resolve ambiguity by removing one of the repos')
 
             init_repo = repo
             init_script = script
@@ -252,11 +252,11 @@ def get_build_environ(build_system):
         raise InitBuildEnvError('Did not find any init-build-env script')
 
     with tempfile.TemporaryDirectory() as temp_dir:
-        script = """#!/bin/bash
+        script = f"""#!/bin/bash
         set -e
-        source %s $1 > /dev/null
+        source {init_script} $1 > /dev/null
         env
-        """ % init_script
+        """
 
         get_bb_env_file = pathlib.Path(temp_dir) / "get_bb_env"
         get_bb_env_file.write_text(script)
@@ -266,7 +266,7 @@ def get_build_environ(build_system):
         env['PATH'] = '/usr/sbin:/usr/bin:/sbin:/bin'
 
         (_, output) = run_cmd([str(get_bb_env_file), get_context().build_dir],
-                              cwd=init_repo.path, env=env, liveupdate=False)
+                              cwd=init_repo.path, env=env)
 
     env = {}
     for line in output.splitlines():
@@ -278,7 +278,7 @@ def get_build_environ(build_system):
 
     conf_env = get_context().config.get_environment()
 
-    env_vars = ['SSTATE_DIR', 'DL_DIR', 'TMPDIR']
+    env_vars = ['SSTATE_DIR', 'SSTATE_MIRRORS', 'DL_DIR', 'TMPDIR']
     env_vars.extend(conf_env)
 
     env.update(conf_env)
@@ -334,17 +334,17 @@ def ssh_cleanup_agent():
     """
         Removes the identities and stops the ssh-agent instance
     """
-    env = get_context().environ
+    ctx = get_context()
     # remove the identities
-    process = Popen(['ssh-add', '-D'], env=env)
-    process.wait()
-    if process.returncode != 0:
+    (ret, _) = run_cmd(['ssh-add', '-D'], cwd=ctx.kas_work_dir,
+                       env=ctx.environ, fail=False)
+    if ret != 0:
         logging.error('failed to delete SSH identities')
 
     # stop the ssh-agent
-    process = Popen(['ssh-agent', '-k'], env=env)
-    process.wait()
-    if process.returncode != 0:
+    (ret, _) = run_cmd(['ssh-agent', '-k'], cwd=ctx.kas_work_dir,
+                       env=ctx.environ, fail=False)
+    if ret != 0:
         logging.error('failed to stop SSH agent')
 
 
@@ -352,9 +352,11 @@ def ssh_setup_agent(envkeys=None):
     """
         Starts the ssh-agent
     """
-    env = get_context().environ
+    ctx = get_context()
+    env = ctx.environ
     envkeys = envkeys or ['SSH_PRIVATE_KEY', 'SSH_PRIVATE_KEY_FILE']
-    output = os.popen('ssh-agent -s').readlines()
+    (_, output) = run_cmd(['ssh-agent', '-s'], env=env,
+                          cwd=ctx.kas_work_dir)
     for line in output:
         matches = re.search(r"(\S+)\=(\S+)\;", line)
         if matches:
@@ -405,15 +407,15 @@ def ssh_no_host_key_check():
 
 def setup_parser_common_args(parser):
     parser.add_argument('config',
-                        help='Config file, using .config.yaml in KAS_WORK_DIR '
-                        'if none is specified',
+                        help='Config file(s), separated by colon. Using '
+                        '.config.yaml in KAS_WORK_DIR if none is specified.',
                         nargs='?')
     parser.add_argument('--skip',
                         help='Skip build steps',
                         default=[])
     parser.add_argument('--force-checkout', action='store_true',
-                        help='Always checkout the desired commit/branch of '
-                        'each repository, discarding any local changes')
+                        help='Always checkout the desired commit/branch/tag '
+                        'of each repository, discarding any local changes')
     parser.add_argument('--update', action='store_true',
                         help='Pull new upstream changes to the desired '
                         'branch even if it is already checked out locally')
