@@ -27,6 +27,7 @@ import re
 import os
 import sys
 import logging
+import shutil
 from urllib.parse import urlparse
 from tempfile import TemporaryDirectory
 from .context import get_context
@@ -76,70 +77,82 @@ class Repo:
         Represents a repository in the kas configuration.
     """
 
-    def __init__(self, name, url, path, commit, branch, refspec, layers,
+    def __init__(self, name, url, path, commit, tag, branch, refspec, layers,
                  patches, disable_operations):
         self.name = name
         self.url = url
         self.path = path
         self.commit = commit
+        self.tag = tag
         self.branch = branch
         self.refspec = refspec
         self._layers = layers
         self._patches = patches
         self.operations_disabled = disable_operations
 
-    def __getattr__(self, item):
-        if item == 'layers':
-            return [os.path.join(self.path, layer).rstrip(os.sep + '.')
-                    for layer in self._layers]
-        elif item == 'qualified_name':
-            url = urlparse(self.url)
-            return ('{url.netloc}{url.path}'
-                    .format(url=url)
-                    .replace('@', '.')
-                    .replace(':', '.')
-                    .replace('/', '.')
-                    .replace('*', '.'))
-        elif item == 'effective_url':
-            mirrors = os.environ.get('KAS_PREMIRRORS', '')
-            for mirror in mirrors.split('\n'):
-                try:
-                    expr, subst = mirror.split()
-                    if re.match(expr, self.url):
-                        return re.sub(expr, subst, self.url)
-                except ValueError:
-                    continue
-            return self.url
-        elif item == 'revision':
-            if self.commit:
-                return self.commit
-            branch = self.branch or self.refspec
-            if not branch:
-                return None
+    @property
+    def layers(self):
+        return [os.path.join(self.path, layer).rstrip(os.sep + '.')
+                for layer in self._layers]
+
+    @property
+    def qualified_name(self):
+        url = urlparse(self.url)
+        return (f'{url.netloc}{url.path}'
+                .replace('@', '.')
+                .replace(':', '.')
+                .replace('/', '.')
+                .replace('*', '.'))
+
+    @property
+    def effective_url(self):
+        mirrors = os.environ.get('KAS_PREMIRRORS', '')
+        for mirror in mirrors.split('\n'):
+            try:
+                expr, subst = mirror.split()
+                if re.match(expr, self.url):
+                    return re.sub(expr, subst, self.url)
+            except ValueError:
+                continue
+        return self.url
+
+    @property
+    def revision(self):
+        if self.commit:
+            return self.commit
+        if self.tag:
+            (_, output) = run_cmd(self.resolve_tag_cmd(),
+                                  cwd=self.path, fail=False)
+            if output:
+                return output.strip()
+            return self.tag
+        branch = self.branch or self.refspec
+        if branch:
             (_, output) = run_cmd(self.resolve_branch_cmd(),
                                   cwd=self.path, fail=False)
             if output:
                 return output.strip()
             return branch
-
-        # Default behaviour
-        raise AttributeError
+        return None
 
     def __str__(self):
-        if self.commit and self.branch:
-            return '%s:%s(%s) %s %s' % (self.url, self.commit, self.branch,
-                                        self.path, self._layers)
-        return '%s:%s %s %s' % (self.url,
-                                self.commit or self.branch or self.refspec,
-                                self.path, self._layers)
+        if self.commit and (self.tag or self.branch):
+            refspec = f'{self.commit}({self.tag or self.branch})'
+        else:
+            refspec = self.commit or self.tag or self.branch or self.refspec
+
+        return f'{self.url}:{refspec} ' \
+               f'{self.path} {self._layers}'
 
     __legacy_refspec_warned__ = []
+    __no_commit_tag_warned__ = []
 
     @staticmethod
     def factory(name, repo_config, repo_defaults, repo_fallback_path,
                 repo_overrides={}):
         """
             Returns a Repo instance depending on params.
+            This factory function is referential transparent.
         """
         layers_dict = repo_config.get('layers', {'': None})
         layers = list(filter(lambda x, laydict=layers_dict:
@@ -159,44 +172,52 @@ class Repo:
             }
             if this_patch['repo'] is None:
                 raise PatchMappingError(
-                    'No repo specified for patch entry "{}" and no '
-                    'default repo specified.'.format(p))
+                    f'No repo specified for patch entry "{p}" and no '
+                    'default repo specified.')
 
             patches.append(this_patch)
 
         url = repo_config.get('url', None)
         name = repo_config.get('name', name)
-        typ = repo_config.get('type', 'git')
+        repo_type = repo_config.get('type', 'git')
         commit = repo_config.get('commit', None)
+        tag = repo_config.get('tag', repo_defaults.get('tag', None))
         branch = repo_config.get('branch', repo_defaults.get('branch', None))
         refspec = repo_config.get('refspec',
                                   repo_defaults.get('refspec', None))
-        if commit is None and branch is None and refspec is None \
-                and url is not None:
-            raise RepoRefError('No commit or branch specified for repository '
-                               '"{}". This is only allowed for local '
-                               'repositories.'.format(name))
+        if commit is None and tag is None and branch is None \
+                and refspec is None and url is not None:
+            raise RepoRefError('No commit, tag or branch specified for '
+                               f'repository "{name}". This is only allowed '
+                               'for local repositories.')
         if refspec is None:
             commit = repo_overrides.get('commit', commit)
+            if commit and get_context().update:
+                logging.warning(f'Update of "{name}" requested, but repo is '
+                                'pinned to a fixed commit. Not updating.')
         else:
             if name not in Repo.__legacy_refspec_warned__:
-                logging.warning('Using deprecated refspec for repository '
-                                '"%s". You should migrate to commit/branch.',
+                logging.warning('Using deprecated refspec for repository "%s".'
+                                ' You should migrate to commit/tag/branch.',
                                 name)
                 Repo.__legacy_refspec_warned__.append(name)
-            if commit is not None or branch is not None:
-                raise RepoRefError('Unsupported mixture of legacy refspec '
-                                   'and commit/branch for repository "{}"'
-                                   .format(name))
+            if commit is not None or tag is not None or branch is not None:
+                raise RepoRefError(
+                    'Unsupported mixture of legacy refspec and '
+                    f'commit/tag/branch for repository "{name}"')
             refspec = repo_overrides.get('commit', refspec)
+        if tag and not commit:
+            if name not in Repo.__no_commit_tag_warned__:
+                logging.warning('Using tag without commit for repository '
+                                '"%s" is unsafe as tags are mutable.', name)
+                Repo.__no_commit_tag_warned__.append(name)
         path = repo_config.get('path', None)
         disable_operations = False
 
         if path is None:
             if url is None:
                 path = Repo.get_root_path(repo_fallback_path)
-                logging.info('Using %s as root for repository %s', path,
-                             name)
+                logging.info('Using %s as root for repository %s', path, name)
             else:
                 path = os.path.join(get_context().kas_work_dir, name)
         elif not os.path.isabs(path):
@@ -208,13 +229,17 @@ class Repo:
             url = path
             disable_operations = True
 
-        if typ == 'git':
-            return GitRepo(name, url, path, commit, branch, refspec, layers,
-                           patches, disable_operations)
-        if typ == 'hg':
-            return MercurialRepo(name, url, path, commit, branch, refspec,
+        if repo_type == 'git':
+            return GitRepo(name, url, path, commit, tag, branch, refspec,
+                           layers, patches, disable_operations)
+        if repo_type == 'hg':
+            if not shutil.which('hg'):
+                raise UnsupportedRepoTypeError(
+                    'hg is required for Mercurial repositories')
+            return MercurialRepo(name, url, path, commit, tag, branch, refspec,
                                  layers, patches, disable_operations)
-        raise UnsupportedRepoTypeError('Repo type "%s" not supported.' % typ)
+        raise UnsupportedRepoTypeError(f'Repo type "{repo_type}" '
+                                       'not supported.')
 
     @staticmethod
     def get_root_path(path, fallback=True):
@@ -222,12 +247,12 @@ class Repo:
             Checks if path is under version control and returns its root path.
         """
         (ret, output) = run_cmd(['git', 'rev-parse', '--show-toplevel'],
-                                cwd=path, fail=False, liveupdate=False)
+                                cwd=path, fail=False)
         if ret == 0:
             return output.strip()
 
         (ret, output) = run_cmd(['hg', 'root'],
-                                cwd=path, fail=False, liveupdate=False)
+                                cwd=path, fail=False)
         if ret == 0:
             return output.strip()
 
@@ -270,43 +295,41 @@ class RepoImpl(Repo):
                                   self.qualified_name)
 
         if not os.path.exists(self.path):
+            logging.info('Cloning repository %s', self.name)
             os.makedirs(os.path.dirname(self.path), exist_ok=True)
             (retc, _) = await run_cmd_async(
                 self.clone_cmd(sdir, createref=False),
                 cwd=get_context().kas_work_dir)
 
-            logging.info('Repository %s cloned', self.name)
-
         # Make sure the remote origin is set to the value
         # in the kas file to avoid surprises
         try:
             (retc, output) = await run_cmd_async(
                 self.set_remote_url_cmd(),
-                cwd=self.path,
-                liveupdate=False)
+                cwd=self.path)
         except NotImplementedError:
             logging.warning('Repo implementation does not support changing '
                             'the remote url.')
 
         # take what came out of clone and stick to that forever
-        if self.commit is None and self.branch is None \
+        if self.commit is None and self.tag is None and self.branch is None \
            and self.refspec is None:
             return 0
 
         if not get_context().update:
-            # Do commit/branch/refspec exist in the current repository?
+            # Do commit/tag/branch/refspec exist in the current repository?
             (retc, output) = await run_cmd_async(self.contains_refspec_cmd(),
                                                  cwd=self.path,
-                                                 fail=False,
-                                                 liveupdate=False)
+                                                 fail=False)
             if retc == 0:
                 logging.info('Repository %s already contains %s as %s',
                              self.name,
-                             self.commit or self.branch or self.refspec,
+                             self.commit or self.tag or self.branch
+                             or self.refspec,
                              output.strip())
                 return retc
 
-        # Try to fetch if commit/branch/refspec is missing or if --update
+        # Try to fetch if commit/tag/branch/refspec is missing or if --update
         # argument was passed
         (retc, output) = await run_cmd_async(self.fetch_cmd(),
                                              cwd=self.path,
@@ -323,8 +346,8 @@ class RepoImpl(Repo):
             Checks out the correct revision of the repo.
         """
         if self.operations_disabled \
-            or (self.commit is None and self.branch is None
-                and self.refspec is None):
+            or (self.commit is None and self.tag is None
+                and self.branch is None and self.refspec is None):
             return
 
         if not get_context().force_checkout:
@@ -336,24 +359,55 @@ class RepoImpl(Repo):
                 logging.warning('Repo %s is dirty - no checkout', self.name)
                 return
 
-        if self.commit:
+        if self.tag and self.branch:
+            raise RepoRefError(
+                f'Both tag "{self.tag}" and branch "{self.branch}" '
+                f'cannot be specified for repository "{self.name}"')
+
+        if self.tag:
+            (retc, output) = run_cmd(self.resolve_tag_cmd(),
+                                     cwd=self.path,
+                                     fail=False)
+            if retc:
+                raise RepoRefError(f'Tag "{self.tag}" cannot be found '
+                                   f'in repository "{self.name}"')
+
+            if self.commit and output.strip() != self.commit:
+                # Ensure provided commit and tag match
+                raise RepoRefError(f'Provided tag "{self.tag}" does not match '
+                                   f'provided commit "{self.commit}" in '
+                                   f'repository "{self.name}", aborting!')
+            desired_ref = output.strip()
+            is_branch = False
+        elif self.branch:
+            (retc, output) = run_cmd(self.resolve_branch_cmd(),
+                                     cwd=self.path,
+                                     fail=False)
+            if retc:
+                raise RepoRefError(
+                    f'Branch "{self.branch}" cannot be found '
+                    f'in repository "{self.name}"')
+            if self.commit:
+                (_, output) = run_cmd(self.branch_contains_ref(),
+                                      cwd=self.path,
+                                      fail=False)
+                if not output.strip():
+                    raise RepoRefError(
+                        f'Branch "{self.branch}" in '
+                        f'repository "{self.name}" does not contain '
+                        f'commit "{self.commit}"')
+
+            desired_ref = self.commit or output.strip()
+            is_branch = True
+        elif self.commit:
             desired_ref = self.commit
             is_branch = False
         else:
-            (_, output) = run_cmd(self.resolve_branch_cmd(),
-                                  cwd=self.path, fail=False)
-            if output:
-                desired_ref = output.strip()
-                is_branch = True
-            elif self.branch:
-                raise RepoRefError(
-                    'Branch "{}" cannot be found in repository {}'
-                    .format(self.branch, self.name))
-            else:
-                desired_ref = self.refspec
-                is_branch = False
+            desired_ref = self.refspec
+            is_branch = False
 
         run_cmd(self.checkout_cmd(desired_ref, is_branch), cwd=self.path)
+        logging.info(f'Repository {self.name} checked out to {desired_ref}')
 
     async def apply_patches_async(self):
         """
@@ -372,10 +426,10 @@ class RepoImpl(Repo):
                                                             None)
 
             if not other_repo:
-                raise PatchMappingError(
-                    'Could not find referenced repo. '
-                    '(missing repo: {}, repo: {}, patch entry: {})'
-                    .format(patch['repo'], self.name, patch['id']))
+                raise PatchMappingError('Could not find referenced repo. '
+                                        f'(missing repo: {patch["repo"]}, '
+                                        f'repo: {self.name}, '
+                                        f'patch entry: {patch["id"]})')
 
             path = os.path.join(other_repo.path, patch['path'])
             cmd = []
@@ -396,8 +450,8 @@ class RepoImpl(Repo):
             else:
                 raise PatchFileNotFound(
                     'Could not find patch. '
-                    '(patch path: {}, repo: {}, patch entry: {})'
-                    .format(path, self.name, patch['id']))
+                    f'(patch path: {path}, repo: {self.name}, patch '
+                    f'entry: {patch["id"]})')
 
         for (path, patch_id) in my_patches:
             cmd = self.apply_patches_file_cmd(path)
@@ -405,10 +459,9 @@ class RepoImpl(Repo):
                 cmd, cwd=self.path, fail=False)
             if retc:
                 raise PatchApplyError(
-                    'Could not apply patch. Please fix repos and '
-                    'patches. (patch path: {}, repo: {}, patch '
-                    'entry: {}, vcs output: {})'
-                    .format(path, self.name, patch_id, output))
+                    'Could not apply patch. Please fix repos and patches. '
+                    f'(patch path: {path}, repo: {self.name}, patch '
+                    f'entry: {patch_id}, vcs output: {output})')
 
             logging.info('Patch applied. '
                          '(patch path: %s, repo: %s, patch entry: %s)',
@@ -418,17 +471,15 @@ class RepoImpl(Repo):
             (retc, output) = await run_cmd_async(
                 cmd, cwd=self.path, fail=False)
             if retc:
-                raise PatchApplyError(
-                    'Could not add patched files. repo: {}, vcs output: {})'
-                    .format(self.name, output))
+                raise PatchApplyError('Could not add patched files. repo: '
+                                      f'{self.name}, vcs output: {output})')
 
             cmd = self.commit_cmd()
             (retc, output) = await run_cmd_async(
                 cmd, cwd=self.path, fail=False)
             if retc:
-                raise PatchApplyError(
-                    'Could not commit patch changes. repo: {}, vcs output: {})'
-                    .format(self.name, output))
+                raise PatchApplyError('Could not commit patch changes. repo: '
+                                      f'{self.name}, vcs output: {output})')
 
         return 0
 
@@ -438,9 +489,9 @@ class GitRepo(RepoImpl):
         Provides the git functionality for a Repo.
     """
 
-    def remove_ref_prefix(self, branch):
+    def remove_ref_prefix(self, ref):
         ref_prefix = 'refs/'
-        return branch[branch.startswith(ref_prefix) and len(ref_prefix):]
+        return ref[ref.startswith(ref_prefix) and len(ref_prefix):]
 
     def add_cmd(self):
         return ['git', 'add', '-A']
@@ -463,16 +514,17 @@ class GitRepo(RepoImpl):
         branch = self.branch or self.refspec
         if branch and branch.startswith('refs/'):
             branch = 'remotes/origin/' + self.remove_ref_prefix(branch)
-        return ['git', 'cat-file', '-t', self.commit or branch]
+        return ['git', 'cat-file', '-t', self.commit or self.tag or branch]
 
     def fetch_cmd(self):
         cmd = ['git', 'fetch', '-q']
+        if self.tag:
+            cmd.append('--tags')
+
         branch = self.branch or self.refspec
         if branch and branch.startswith('refs/'):
-            cmd.extend(['origin',
-                        '+' + branch
-                        + ':refs/remotes/origin/'
-                        + self.remove_ref_prefix(branch)])
+            branch = self.remove_ref_prefix(branch)
+            cmd.extend(['origin', f'+{branch}:refs/remotes/origin/{branch}'])
 
         return cmd
 
@@ -480,10 +532,15 @@ class GitRepo(RepoImpl):
         return ['git', 'status', '-s']
 
     def resolve_branch_cmd(self):
-        return ['git', 'rev-parse', '--verify', '-q',
-                'origin/{branch}'.
-                format(branch=self.remove_ref_prefix(
-                    self.branch or self.refspec))]
+        refspec = self.remove_ref_prefix(self.branch or self.refspec)
+        return ['git', 'rev-parse', '--verify', '-q', f'origin/{refspec}']
+
+    def resolve_tag_cmd(self):
+        return ['git', 'rev-list', '-n', '1', self.remove_ref_prefix(self.tag)]
+
+    def branch_contains_ref(self):
+        return ['git', 'branch', f'origin/{self.branch}',
+                '-r', '--contains', self.commit]
 
     def checkout_cmd(self, desired_ref, is_branch):
         cmd = ['git', 'checkout', '-q', self.remove_ref_prefix(desired_ref)]
@@ -496,10 +553,9 @@ class GitRepo(RepoImpl):
         return cmd
 
     def prepare_patches_cmd(self):
-        branch = self.branch or self.refspec
-        return ['git', 'checkout', '-q', '-B',
-                'patched-{refspec}'.
-                format(refspec=self.commit or self.remove_ref_prefix(branch))]
+        refspec = self.commit \
+            or self.remove_ref_prefix(self.tag or self.branch or self.refspec)
+        return ['git', 'checkout', '-q', '-B', f'patched-{refspec}']
 
     def apply_patches_file_cmd(self, path):
         return ['git', 'apply', '--whitespace=nowarn', path]
@@ -526,7 +582,8 @@ class MercurialRepo(RepoImpl):
         return ['hg', 'commit', '--user', 'kas <kas@example.com>', '-m', 'msg']
 
     def contains_refspec_cmd(self):
-        return ['hg', 'log', '-r', self.commit or self.branch or self.refspec]
+        return ['hg', 'log', '-r', self.commit or self.tag or self.branch
+                or self.refspec]
 
     def fetch_cmd(self):
         return ['hg', 'pull']
@@ -535,8 +592,18 @@ class MercurialRepo(RepoImpl):
         return ['hg', 'diff']
 
     def resolve_branch_cmd(self):
-        return ['hg', 'identify', '--id', '-r', self.branch or self.refspec,
-                'default']
+        if self.branch:
+            return ['hg', 'identify', '--id', '-r',
+                    f'limit(heads(branch({self.branch})))']
+        else:
+            return ['hg', 'identify', '--id', '-r', self.refspec]
+
+    def resolve_tag_cmd(self):
+        refspec = self.tag or self.refspec
+        return ['hg', 'identify', '--id', '-r', f'tag({refspec})']
+
+    def branch_contains_ref(self):
+        return ['hg', 'log', '-r', self.commit, '-b', self.branch]
 
     def checkout_cmd(self, desired_ref, is_branch):
         cmd = ['hg', 'checkout', desired_ref]
@@ -545,9 +612,8 @@ class MercurialRepo(RepoImpl):
         return cmd
 
     def prepare_patches_cmd(self):
-        refspec = self.commit or self.branch or self.refspec
-        return ['hg', 'branch', '-f',
-                'patched-{refspec}'.format(refspec=refspec)]
+        refspec = self.commit or self.tag or self.branch or self.refspec
+        return ['hg', 'branch', '-f', f'patched-{refspec}']
 
     def apply_patches_file_cmd(self, path):
         return ['hg', 'import', '--no-commit', path]
