@@ -1,6 +1,6 @@
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2019
+# Copyright (c) Siemens AG, 2019-2024
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,12 +20,23 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import os
-
 import pytest
 
+ENVVARS_KAS = [
+    'KAS_WORK_DIR',
+    'KAS_BUILD_DIR',
+    'KAS_REPO_REF_DIR',
+    'KAS_DISTRO',
+    'KAS_MACHINE',
+    'KAS_TARGET',
+    'KAS_TASK',
+    'KAS_PREMIRRORS',
+    'SSH_AUTH_SOCK',
+]
+
 
 @pytest.fixture
-def changedir():
-    yield
-    os.chdir(os.path.join(os.path.dirname(__file__), '..'))
+def monkeykas(monkeypatch):
+    for var in ENVVARS_KAS:
+        monkeypatch.delenv(var, raising=False)
+    yield monkeypatch
